"use strict";

var DateInput = function DateInput(options) {
  this.container = $(options.container);
  this.dayInput = this.container.find(options.daySelector);
  this.monthInput = this.container.find(options.monthSelector);
  this.yearInput = this.container.find(options.yearSelector);
  this.container = this.container;
  options = options || {}; // use selected date or today's date for the month date

  this.monthDate = this.hasSelectedDate() ? this.getInputDate() : new Date(); // use the selected date or set to null because there is no selected date

  this.selectedDate = this.hasSelectedDate() ? this.getInputDate() : null;
  this.setupOptions(options);
  this.setupKeys();
  this.setupMonthNames();
  this.setupShortMonthNames();
  this.setupToggleButton();
  this.setupCalendarHtml();
  this.updateSelectedDateStatusBox();
};

DateInput.prototype.setupCalendarHtml = function () {
  this.calendar = $('<div class="datepicker hidden" aria-label="date picker" role="group" tabindex="-1">');
  this.calendar.html(this.getCalendarHtml(this.monthDate.getFullYear(), this.monthDate.getMonth()));
  this.addEventListeners();
  this.container.append(this.calendar);
};

DateInput.prototype.updateSelectedDateStatusBox = function () {
  var status = this.selectedDate ? 'The currently selected date is ' + this.getFullDateString(this.selectedDate) : '';
  this.container.find('.datepicker-selected-date').html(status);
};

DateInput.prototype.getFullDateString = function (d) {
  return d.getDate() + ' ' + this.monthNames[d.getMonth()] + ' ' + d.getFullYear();
};

DateInput.prototype.getSelectedDateElementId = function (d) {
  return this.dayInput[0].id + '-selected-date';
};

DateInput.prototype.getCalendarHtml = function (year, month) {
  var html = '<div id="' + this.getSelectedDateElementId() + '" class="datepicker-selected-date visually-hidden"></div>';
  html += '<div class="datepicker-actions">';
  html += '<button type="button" aria-label="previous month"><svg aria-hidden="true" focusable="false" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" clip-rule="evenodd" d="M15.4 7.4L14 6L8 12L14 18L15.4 16.6L10.8 12L15.4 7.4Z" fill="currentColor"/></svg></button>';
  html += '<div role="status" aria-live="polite" class="visually-hidden">';
  html += this.monthNames[month] + " " + year;
  html += '</div>';
  html += '<select>';
  html += this.getSelectOptionsHtml(year, month);
  html += '</select>';
  html += '<button type="button" aria-label="next month"><svg aria-hidden="true" focusable="false" width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M8.59 16.59L10 18L16 12L10 6L8.59 7.41L13.17 12L8.59 16.59Z" fill="currentColor"/></svg></button>';
  html += '</div>';
  html += '<table role="grid">';
  html += '<thead>';
  html += '<tr>';
  html += '<th><span aria-hidden="true">Mo</span><span class="visually-hidden">Monday</span></th>';
  html += '<th><span aria-hidden="true">Tu</span><span class="visually-hidden">Tuesday</span></th>';
  html += '<th><span aria-hidden="true">We</span><span class="visually-hidden">Wedsday</span></th>';
  html += '<th><span aria-hidden="true">Th</span><span class="visually-hidden">Thursday</span></th>';
  html += '<th><span aria-hidden="true">Fr</span><span class="visually-hidden">Friday</span></th>';
  html += '<th><span aria-hidden="true">Sa</span><span class="visually-hidden">Saturday</span></th>';
  html += '<th><span aria-hidden="true">Su</span><span class="visually-hidden">Sunday</span></th>';
  html += '</tr>';
  html += '</thead>';
  html += '<tbody>';
  html += this.getCalendarTableRows(month, year);
  html += '</tbody>';
  html += '</table>';
  return html;
};

DateInput.prototype.getSelectOptionsHtml = function (year, month) {
  var html = '';
  var d = new Date(year, month, 1);
  d.setMonth(d.getMonth() - 24);

  for (var i = -1; i < 48; i++) {
    d.setMonth(d.getMonth() + 1);
    var selected = d.getMonth() == month && d.getFullYear() == year ? 'selected' : '';
    html += '<option value="' + d.toString() + '" ' + selected + '>';
    html += this.shortMonthNames[d.getMonth()] + ' ' + d.getFullYear();
    html += '</option>';
  }

  return html;
};

DateInput.prototype.onSelectChange = function (e) {
  this.monthDate = new Date($(e.currentTarget).val());
  this.updateCalendarHtml(this.monthDate.getFullYear(), this.monthDate.getMonth());
};

DateInput.prototype.hasSelectedDate = function () {
  return this.dayInput[0].value && this.monthInput[0].value && this.yearInput[0].value;
};

DateInput.prototype.getInputDate = function () {
  return new Date(parseInt(this.yearInput[0].value, 10), parseInt(this.monthInput[0].value, 10) - 1, parseInt(this.dayInput[0].value, 10));
};

DateInput.prototype.setupMonthNames = function () {
  this.monthNames = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
};

DateInput.prototype.setupShortMonthNames = function () {
  this.shortMonthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
};

DateInput.prototype.setupOptions = function (options) {
  var defaults = {};

  defaults.dateStart = function () {
    var d = new Date();
    d.setYear(d.getFullYear() - 1);
    return d;
  }();

  defaults.currentDate = function () {
    var d = new Date();
    d.setHours(0, 0, 0, 0);
    return d;
  }();

  this.options = $.extend(defaults, options);
};

DateInput.prototype.getFirstDateOfMonth = function (month, year) {
  var d = new Date();
  d.setFullYear(year, month, 1, 0);
  d.setHours(0, 0, 0, 0);
  return d;
};

DateInput.prototype.getCalendarTableRows = function (month, year) {
  var html = '';
  var now = new Date();
  now.setHours(0, 0, 0, 0); // get first day of month and first week day

  var first_day = new Date(year, month, 1),
      first_day_weekday = first_day.getDay() == 0 ? 7 : first_day.getDay(); // find number of days in month

  var month_length = new Date(year, month + 1, 0).getDate(),
      previous_month_length = new Date(year, month, 0).getDate(); // define default day variables

  var day = 1,
      // current month days
  prev = 1,
      // previous month days
  next = 1; // next month days

  html += '<tr class="week">'; // weeks loop (rows)

  for (var i = 0; i < 9; i++) {
    // weekdays loop (cells)
    for (var j = 1; j <= 7; j++) {
      if (day <= month_length && (i > 0 || j >= first_day_weekday)) {
        var d = new Date(year, month, day);
        var focusableDate = null;

        if (this.selectedDate && this.selectedDate.getMonth() === d.getMonth() && this.selectedDate.getFullYear() === d.getFullYear()) {
          // selected date
          focusableDate = this.selectedDate;
        } else if (now.getMonth() === d.getMonth() && now.getFullYear() === d.getFullYear()) {
          // today
          focusableDate = now;
        } else {
          // 1st of month
          focusableDate = first_day;
        }

        html += this.getCellHtml(d, {
          today: d.getTime() === now.getTime() ? true : false,
          focusable: focusableDate.getTime() === d.getTime() ? true : false,
          selected: this.selectedDate && this.selectedDate.getTime() === d.getTime() ? true : false
        });
        day++;
      } else {
        if (day <= month_length) {
          // previous month
          html += '<td class="day other-month">';
          html += new Date(year, month - 1, previous_month_length - first_day_weekday + prev + 1).getDate();
          html += '</td>';
          prev++;
        } else {
          // next month
          html += '<td class="day other-month">'; // html += next;

          html += new Date(year, month + 1, next).getDate();
          html += '</td>';
          next++;
        }
      }
    } // stop making rows if it's the end of month


    if (day > month_length) {
      html += '</tr>';
      break;
    } else {
      html += '</tr><tr class="week">';
    }
  }

  return html;
};

DateInput.prototype.getCellHtml = function (date, options) {
  var label = date.getDate() + ' ' + this.monthNames[date.getMonth()] + ' ' + date.getFullYear();
  var tdClass = '';

  if (options.today) {
    tdClass += ' datepicker-day-isToday';
  }

  if (options.focusable) {
    tdClass += ' datepicker-day-isFocused';
  }

  var html = '';
  html += '<td';

  if (options.focusable) {
    html += ' tabindex="0" ';
  } else {
    html += ' tabindex="-1" ';
  }

  if (options.selected) {
    html += ' aria-selected="true" ';
  } else {
    html += ' aria-selected="false" ';
  } // html += ' aria-label="'+  label + '" ';


  html += ' role="gridcell" ';
  html += ' data-date="' + date.toString() + '" ';
  html += ' class="' + tdClass + '" ';
  html += '>';

  if (options.today) {// html += '<span class="datepicker-today">Today</span>';
  }

  html += '<span aria-hidden="true">';
  html += date.getDate();
  html += '</span>';
  html += '<span class="visually-hidden">' + label + '</a>';
  html += '</td>';
  return html;
};

DateInput.prototype.addEventListeners = function () {
  this.calendar.on('click', 'button:first-child', $.proxy(this, 'onPreviousClick'));
  this.calendar.on('click', 'button:last-child', $.proxy(this, 'onNextClick'));
  this.calendar.on('click', '[role=gridcell]', $.proxy(this, 'onCellClick'));
  this.calendar.on('keydown', '[role=gridcell]', $.proxy(this, 'onCellKeyDown'));
  this.calendar.on('change', 'select', $.proxy(this, 'onSelectChange'));
  this.calendar.on('keydown', $.proxy(this, 'onCalendarKeyDown'));
};

DateInput.prototype.setupToggleButton = function () {
  this.toggleButton = $('<button type="button" aria-haspopup="true" aria-labelledby="btnLabel" aria-describedby="' + this.getSelectedDateElementId() + '" class="datepicker-toggle-button btn icon-btn"><svg width="14" height="15" viewBox="0 0 14 15" aria-hidden="true" focusable="false"><path d="M11.8125 2.5H10.5V1.07812C10.5 0.914062 10.3359 0.75 10.1719 0.75H9.95312C9.76172 0.75 9.625 0.914062 9.625 1.07812V2.5H4.375V1.07812C4.375 0.914062 4.21094 0.75 4.04688 0.75H3.82812C3.63672 0.75 3.5 0.914062 3.5 1.07812V2.5H2.1875C1.44922 2.5 0.875 3.10156 0.875 3.8125V13.4375C0.875 14.1758 1.44922 14.75 2.1875 14.75H11.8125C12.5234 14.75 13.125 14.1758 13.125 13.4375V3.8125C13.125 3.10156 12.5234 2.5 11.8125 2.5ZM2.1875 3.375H11.8125C12.0312 3.375 12.25 3.59375 12.25 3.8125V5.125H1.75V3.8125C1.75 3.59375 1.94141 3.375 2.1875 3.375ZM11.8125 13.875H2.1875C1.94141 13.875 1.75 13.6836 1.75 13.4375V6H12.25V13.4375C12.25 13.6836 12.0312 13.875 11.8125 13.875ZM4.92188 9.5C5.08594 9.5 5.25 9.36328 5.25 9.17188V8.07812C5.25 7.91406 5.08594 7.75 4.92188 7.75H3.82812C3.63672 7.75 3.5 7.91406 3.5 8.07812V9.17188C3.5 9.36328 3.63672 9.5 3.82812 9.5H4.92188ZM7.54688 9.5C7.71094 9.5 7.875 9.36328 7.875 9.17188V8.07812C7.875 7.91406 7.71094 7.75 7.54688 7.75H6.45312C6.26172 7.75 6.125 7.91406 6.125 8.07812V9.17188C6.125 9.36328 6.26172 9.5 6.45312 9.5H7.54688ZM10.1719 9.5C10.3359 9.5 10.5 9.36328 10.5 9.17188V8.07812C10.5 7.91406 10.3359 7.75 10.1719 7.75H9.07812C8.88672 7.75 8.75 7.91406 8.75 8.07812V9.17188C8.75 9.36328 8.88672 9.5 9.07812 9.5H10.1719ZM7.54688 12.125C7.71094 12.125 7.875 11.9883 7.875 11.7969V10.7031C7.875 10.5391 7.71094 10.375 7.54688 10.375H6.45312C6.26172 10.375 6.125 10.5391 6.125 10.7031V11.7969C6.125 11.9883 6.26172 12.125 6.45312 12.125H7.54688ZM4.92188 12.125C5.08594 12.125 5.25 11.9883 5.25 11.7969V10.7031C5.25 10.5391 5.08594 10.375 4.92188 10.375H3.82812C3.63672 10.375 3.5 10.5391 3.5 10.7031V11.7969C3.5 11.9883 3.63672 12.125 3.82812 12.125H4.92188ZM10.1719 12.125C10.3359 12.125 10.5 11.9883 10.5 11.7969V10.7031C10.5 10.5391 10.3359 10.375 10.1719 10.375H9.07812C8.88672 10.375 8.75 10.5391 8.75 10.7031V11.7969C8.75 11.9883 8.88672 12.125 9.07812 12.125H10.1719Z" fill="currentColor"/></svg> <span id="btnLabel" hidden>Choose</span></button>');
  this.container.append(this.toggleButton);
  this.toggleButton.on('click', $.proxy(this, 'onToggleButtonClick'));
};

DateInput.prototype.onToggleButtonClick = function () {
  if (this.toggleButton.attr('aria-expanded') == 'true') {
    this.hide();
  } else {
    if (this.hasSelectedDate()) {
      this.selectedDate = this.getInputDate();
      this.updateCalendarHtml(this.selectedDate.getFullYear(), this.selectedDate.getMonth());
    } else {
      this.updateCalendarHtml(this.monthDate.getFullYear(), this.monthDate.getMonth());
    }

    this.show();
    this.calendar.focus();
  }
};

DateInput.prototype.hide = function () {
  this.calendar.addClass('hidden');
  this.toggleButton.attr('aria-expanded', 'false');
};

DateInput.prototype.show = function () {
  this.calendar.removeClass('hidden');
  this.toggleButton.attr('aria-expanded', 'true');
};

DateInput.prototype.setupKeys = function () {
  this.keys = {
    tab: 9,
    enter: 13,
    esc: 27,
    space: 32,
    pageup: 33,
    pagedown: 34,
    end: 35,
    home: 36,
    left: 37,
    up: 38,
    right: 39,
    down: 40
  };
};

DateInput.prototype.onCalendarKeyDown = function (e) {
  switch (e.keyCode) {
    case this.keys.esc:
      this.hide();
      this.toggleButton.focus();
      break;
  }
};

DateInput.prototype.onCellKeyDown = function (e) {
  switch (e.keyCode) {
    case this.keys.down:
      this.onDayDownPressed(e);
      break;

    case this.keys.up:
      this.onDayUpPressed(e);
      break;

    case this.keys.left:
      this.onDayLeftPressed(e);
      break;

    case this.keys.right:
      this.onDayRightPressed(e);
      break;

    case this.keys.space:
    case this.keys.enter:
      this.onDaySpacePressed(e);
      break;
  }
};

DateInput.prototype.onCellClick = function (e) {
  var d = new Date($(e.currentTarget).attr('data-date'));
  this.updateInputValues(d);
  this.selectedDate = d;
  this.selectDate(d);
  this.hide();
  this.updateSelectedDateStatusBox();
  this.toggleButton.focus();
};

DateInput.prototype.onPreviousClick = function (e) {
  this.showPreviousMonth();
};

DateInput.prototype.onNextClick = function (e) {
  this.showNextMonth();
  console.log('i ran');
};

DateInput.prototype.onDaySpacePressed = function (e) {
  e.preventDefault();
  var d = new Date($(e.currentTarget).attr('data-date'));
  this.updateInputValues(d);
  this.selectedDate = d;
  this.selectDate(d);
  this.hide();
  this.updateSelectedDateStatusBox();
  this.toggleButton.focus();
};

DateInput.prototype.selectDate = function (date) {
  this.calendar.find('[aria-selected=true]').attr('aria-selected', 'false');
  this.getDayCell(date).attr('aria-selected', 'true');
};

DateInput.prototype.unhighlightCell = function (date) {
  var cell = this.getDayCell(date);
  cell.attr('tabindex', '-1');
};

DateInput.prototype.highlightCell = function (date) {
  this.unhighlightCell(this.getFocusedCellDate());
  var cell = this.getDayCell(date);
  cell.attr('tabindex', '0');
  cell.focus();
};

DateInput.prototype.getFocusedCell = function () {
  return this.calendar.find('[tabindex="0"]');
};

DateInput.prototype.getFocusedCellDate = function () {
  return new Date(this.getFocusedCell().attr('data-date'));
};

DateInput.prototype.onDayDownPressed = function (e) {
  e.preventDefault();
  var date = this.getFocusedCellDate();
  var newDate = this.getSameDayNextWeek(date);

  if (newDate.getMonth() == date.getMonth()) {
    this.highlightCell(newDate);
  } else {
    this.monthDate = newDate;
    this.updateCalendarHtml(newDate.getFullYear(), newDate.getMonth());
    this.highlightCell(newDate);
  }
};

DateInput.prototype.onDayUpPressed = function (e) {
  e.preventDefault();
  var date = this.getFocusedCellDate();
  var newDate = this.getSameDayLastWeek(date);

  if (newDate.getMonth() == this.monthDate.getMonth()) {
    this.highlightCell(newDate);
  } else {
    this.monthDate = newDate;
    this.updateCalendarHtml(newDate.getFullYear(), newDate.getMonth());
    this.highlightCell(newDate);
  }
};

DateInput.prototype.onDayLeftPressed = function (e) {
  e.preventDefault();
  var date = this.getFocusedCellDate();
  var newDate = this.getPreviousDay(date);

  if (newDate.getMonth() == this.monthDate.getMonth()) {
    this.highlightCell(newDate);
  } else {
    this.monthDate = newDate;
    this.updateCalendarHtml(newDate.getFullYear(), newDate.getMonth());
    this.highlightCell(newDate);
  }
};

DateInput.prototype.onDayRightPressed = function (e) {
  e.preventDefault();
  var date = this.getFocusedCellDate();
  var newDate = this.getNextDay(date);

  if (newDate.getMonth() == this.monthDate.getMonth()) {
    this.highlightCell(newDate);
  } else {
    this.monthDate = newDate;
    this.updateCalendarHtml(newDate.getFullYear(), newDate.getMonth());
    this.highlightCell(newDate);
  }
};

DateInput.prototype.getPreviousDay = function (date) {
  var d = new Date(date);
  d.setDate(d.getDate() - 1);
  return d;
};

DateInput.prototype.getSameDayLastWeek = function (date) {
  var d = new Date(date);
  d.setDate(d.getDate() - 7);
  return d;
};

DateInput.prototype.getNextDay = function (date) {
  var d = new Date(date);
  d.setDate(d.getDate() + 1);
  return d;
};

DateInput.prototype.getSameDayNextWeek = function (date) {
  var d = new Date(date);
  d.setDate(d.getDate() + 7);
  return d;
};

DateInput.prototype.getDayCell = function (date) {
  return this.calendar.find('[data-date="' + date.toString() + '"]');
};

DateInput.prototype.updateCalendarHtml = function (year, month) {
  this.calendar.find('[role=status]').html(this.monthNames[month] + ' ' + year);
  this.calendar.find('select').html(this.getSelectOptionsHtml(year, month));
  this.calendar.find("tbody").html(this.getCalendarTableRows(month, year));
};

DateInput.prototype.updateInputValues = function (date) {
  this.dayInput.val(date.getDate());
  this.monthInput.val(date.getMonth() + 1);
  this.yearInput.val(date.getFullYear());
};

DateInput.prototype.showPreviousMonth = function () {
  var pm = this.getPreviousMonth();
  this.monthDate = pm;
  this.updateCalendarHtml(pm.getFullYear(), pm.getMonth());
};

DateInput.prototype.showNextMonth = function () {
  var nm = this.getNextMonth();
  this.monthDate = nm;
  this.updateCalendarHtml(nm.getFullYear(), nm.getMonth());
};

DateInput.prototype.getPreviousMonth = function () {
  var dayInMs = 86400000;
  var d = new Date(this.monthDate.getFullYear(), this.monthDate.getMonth(), 1);
  d = d.getTime() - dayInMs;
  d = new Date(d);
  d.setDate(1);
  return d;
};

DateInput.prototype.getNextMonth = function () {
  var d = new Date(this.monthDate.getFullYear(), this.monthDate.getMonth());
  d = d.setMonth(d.getMonth() + 1);
  d = new Date(d);
  d.setDate(1);
  return d;
}; // new DateInput({
//         container: $('#date'),
//         daySelector: '.day',
//         monthSelector: '.month',
//         yearSelector: '.year'
//     });
// new DateInput({
//     container: $('#date2'),
//     daySelector: '.day',
//     monthSelector: '.month',
//     yearSelector: '.year'
// });
// new DateInput({
//     container: $('#range-end'),
//     daySelector: '.day',
//     monthSelector: '.month',
//     yearSelector: '.year'
// });